function [C,t,f,c,chirps,sig_chirps,selection,lmax] = chirpgathering(y,wlen,olap,nbfreq,nbcr,CRmax,typef,pfa);
% [C,t,f,c,chirps,chirpsf,sigpoints,lmax,selection,iA] = chirpgathering(y,wlen,olap,nbfreq,nbcr,CRmax,typef,pfa);
% Calculates the signal's chirplet transform. Then iteratively detects
% significant chirplets using the upper bound window and a constant false
% alarm rate. The detected chirplets are then gathered into linear chirps.
%
% Input parameters:
%   
%   y             input data
%   wlen          window length (longer windows are prefered)
%   olap          frame overlap
%   nbfreq        number of frequencies
%   nbcr          number of chirprates
%   CRmax         Chirp Rate Maximum
%   typef         window type used in analysis:
%                'r' = 'Rect'
%                'k' = 'Kaiser',4
%                'h' = 'Hanning'
%                'hm' = 'Hamming'
%                'b' = 'Blackman'
%                'g' = 'Gauss' [DEFAULT]
%   pfa          Probability of false alarm for individual T-F detections
%
% Output variables:
%
%   C           Chirplet transform, indexed in the order f,t,c
%   t           the time values for the time index of C
%   f           the freq values for the freq index of C
%   c           the chirprate values for the chirprate index of C
%   chirps      lists all detected chirps in a matrix. Each row
%                   identifies a detected chirp and consists of 4 values:
%
%                   chirps(indchirp,:) = ...
%                      [start time, initial frequency, end time, chirprate] 
%
%   sig_chirps  A pruned list of chirps that are at least 4*dt in length
%               where dt is the time step in t
%   selection   The TF mask indicating the TF points where a significant
%               chirplet has been detected
%   lmax        The chirprate index for the maximum chirplet at each TF
%               point
%    
% NB1: frequencies are normalized - assumes fs = 1
% NB2: point (t,f) defines the middle point of the chirplet
%
% Code written by Fabien Millioz and Mike Davies, last update 2016. This
% function is part of a software implementation of the Maximum Chirplet
% Transform and its application to the detection of piecewise linear chirp
% signals. Theory and details of the technique can be found in:
%
%   Millioz and Davies, "Sparse detection in the chirplet transform:
%   application to FMCW radar signals," IEEE Transactions on Sig. Proc. 
%   vol 60(6), pp 2800  2813, 2012.
%
% Please reference this paper in any publication that makes use of this
% software. Copyright 2016, University of Edinburgh.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Chirplet transform (complex)                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[C,t,f,c] = chirpletc(y,wlen,olap,nbfreq,nbcr,CRmax,typef);
disp('End of the chirplet computation');
%toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Determination of maximal (upper bound) spectral window           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% NB: for real time implementation calculate off-line and store in memory 
winmax = maxwindow(wlen,nbfreq,CRmax,nbcr,typef);
winmax=winmax/max(winmax);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Determination of chirplets points                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[selection,lmax]=detectionWinmax(C,nbcr,winmax,pfa);
disp('End of determination of signal chirplets')
disp([num2str(sum(sum(selection))), ' chirplets coefficients are selected.'])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    Search of the molecules                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
flag=0; % look for chirps over CRmax?
[chirps,sig_chirps,atoms]=gathering(C,selection,CRmax,nbcr,wlen,t,f,c,lmax,flag);
disp('End of algorithm')



