function [C,t,f,c] = chirpletc(y,wlen,olap,nbfreq,nbcr,CRmax,typef); 
% function [C,t,f,c] = chirpletc(y,wlen,olap,nbfreq,nbcr,CRmax,typef);
% 
% Calculate the Chirplet transform for the input signal y
%
%   input parameters
%   
%   y             input data
%   wlen          window length (longer windows are prefered)
%   olap          frame overlap
%   nbfreq        number of frequencies
%   nbcr          number of chirprates
%   CRmax         Chirp Rate Maximum (+/-)
%   typef         window type used in analysis:
%                'r' = 'Rect'
%                'k' = 'Kaiser',4
%                'h' = 'Hanning'
%                'hm' = 'Hamming'
%                'b' = 'Blackman'
%                'g' = 'Gauss' [DEFAULT]
%
% Output variables:
%
%   C           Chirplet transform, indexed in the order f,t,c
%   t           the time values for the time index of C
%   f           the freq values for the freq index of C
%   c           the chirprate values for the chirprate index of C
%
%
% NB1: we assume fs = 1
% NB2: point (t,f) is the middle point of the chirplet
% NB3: does not assume hermitian symetry
%
% Code written by Fabien Millioz and Mike Davies, last update 2016. This
% function is part of a software implementation of the Maximum Chirplet
% Transform and its application to the detection of piecewise linear chirp
% signals. Theory and details of the technique can be found in:
%
%   Millioz and Davies, "Sparse detection in the chirplet transform:
%   application to FMCW radar signals," IEEE Transactions on Sig. Proc. 
%   vol 60(6), pp 2800  2813, 2012.
%
% Please reference this paper in any publication that makes use of this
% software. Copyright 2016, University of Edinburgh.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Chirplet transform                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if prod(size(y))~=length(y)
  error('Multidimensional signal');
end
if (log2(nbfreq)-floor(log2(nbfreq)))~=0
  fprintf('Better if nbfreq is a power of two\n');
end
if rem(nbcr,2)==0
  fprintf('Warning, even chirprates: no chirprate=0\n');
end

y=y(:);

if nargin==7;
  switch typef 
      case 'r'
    win=tftb_window(wlen,'Rect');
      case 'k'
    win=tftb_window(wlen,'Kaiser',4);
      case 'h'
    win=tftb_window(wlen,'Hanning');
      case 'hm'
    win=tftb_window(wlen,'Hamming');
      case 'b'
    win=tftb_window(wlen,'Blackman');
      case 'g'
    win=tftb_window(wlen,'Gauss');
  end
  else
    win=tftb_window(wlen,'Gauss');
end

win_norm=win; % max(win) must be unity

% test of good detection
disp(['limit chirp step: ', num2str(2/(sum(win)^2))])
disp(['actual chirp step: ', num2str(2*CRmax/(nbcr-1))])

% indices of the window's center
value=(wlen+1)/2;
ind=1;
t=value;
while value<(length(y)-(wlen+1)/2);
  t(ind)=value;
  ind=ind+1;
  value=value+wlen-olap;
end
tcol=length(t);

f=[0:(nbfreq-1)]'/nbfreq;
zeropad=nbfreq;

c=linspace(-CRmax,CRmax,nbcr);
%c=[-10.^(((41:-1:1)-91)/10) 0 10.^(((1:41)-91)/10)];

% Memory allocation -- decreases computation time
C=zeros(length(f),tcol,length(c));

for indcr=1:length(c)
  Lh=(wlen-1)/2;
  win_chirp = win_norm .*exp(-2*pi*i*c(indcr)/2.*((-Lh:Lh).^2))';

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%% Code hacked from GNU tfrstft of the TFBT toolbox %%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  tfr=  zeros (zeropad,tcol) ;
  for  icol=1:tcol,
    ti= t(icol);
    tau=-min([round(zeropad/2)-1,Lh,ti-1]):min([round(zeropad/2)-1,Lh,length(y)-ti]);
    indices= rem(zeropad+tau,zeropad)+1;
    tfr(indices,icol)=y(ti+tau,1).*conj(win_chirp(tau+Lh+1));
  end;
  tfr=fft(tfr);
      
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  %%% End of code GNU tfrstft                         %%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
  
  C(:,:,indcr)=tfr(:,:);
  % C(:,:,indcr)=fftshift(tfr(:,:),1);
end
