% This miniscript provides an example use of the MCT signal detection and
% separation as described in the paper:
%
% Code written by Fabien Millioz and Mike Davies, last update 2016. This
% function is part of a software implementation of the Maximum Chirplet
% Transform and its application to the detection of piecewise linear chirp
% signals. Theory and details of the technique can be found in:
%
%   Millioz and Davies, "Sparse detection in the chirplet transform:
%   application to FMCW radar signals," IEEE Transactions on Sig. Proc. 
%   vol 60(6), pp 2800  2813, 2012.
%
% Please reference this paper in any publication that makes use of this
% software. Copyright 2016, University of Edinburgh.


clear all
close all

% Set up initial design parameters

letter='H'  ;   % Identifier to select signal mixture (A,B,G or H)
wlen=4096-1;    % TF analysis window length
typef = 'h';    % window type used in analysis:
                %  'r' = 'Rect'
                %  'k' = 'Kaiser',4
                %  'h' = 'Hanning'
                %  'hm' = 'Hamming'
                %  'b' = 'Blackman'
                %  'g' = 'Gauss' [DEFAULT if variable not assigned]
nbcr=11         % number of chirp rates to use in chirplet transform (an odd number ensures that we have a zero chirprate
CRmax=2e-6      % Maximum chirp rate
pfa = 1e-3;     % probability of false alarm for signal detecion in each TF bin
fs = 1;         % Sampling frequency (only used in plotting figures)

% Generate 6 individual piece wise linear FMCW signals

[signal1, time]=chirpprofilec(1,[1 500000 1000000 1500000 2000000 2048000; 0.4 0.8 0.4 0.4 0.8 0.7616]);
[signal2, time]=chirpprofilec(1,[1 450000 1000000 1400000 1850000 2048000; 0.2 1 0.2 0.2 1 0.712]);
[signal3, time]=chirpprofilec(1,[1 475000 875000 1225000 1700000 2048000; 0.5 0.6 0.5 0.5 0.6 0.513]);
[signal4, time]=chirpprofilec(1,[1 300000 600000 625000 925000 1225000 1250000 1550000 1850000 1875000 2048000; 0.6 0.7 0.6 0.6 0.7 0.6 0.6 0.7 0.6 0.6 0.657667]);
[signal5, time]=chirpprofilec(1,[1 150000 152500 302500 452500 455000 605000 755000 757500 907500 1057500 1060000 1210000 1360000 1362500 1512500 1662500 1665000 1815000 1965000 1967500 2048000 ; 0.8 0.9 0.8 0.8 0.9 0.8 0.8 0.9 0.8 0.8 0.9 0.8 0.8 0.9 0.8 0.8 0.9 0.8 0.8 0.9 0.8 0.8]);
[signal6, time]=chirpprofilec(1,[1 500000 505000 1005000 1505000 1510000 2010000 2048000; 0.66 0.86 0.66 0.66 0.86 0.66 0.66 0.6752]);

% Generate mixtures of FMCW signals

sigA= signal1/8 + signal2/8 + (randn(size(signal1))+1i*randn(size(signal1))) * sqrt(1/2);  % SNR -21dB
sigB= signal1 + signal2 + (randn(size(signal1))+1i*randn(size(signal1))) * sqrt(1/2);      % SNR -3dB
sigG= signal1/8 + signal2/8 + signal3/8 + signal4/8 + signal5/8 +signal6/8 + (randn(size(signal1))+1i*randn(size(signal1))) * sqrt(1/2); % SNR -21dB
sigH= signal1 + signal2 + signal3 + signal4 + signal5 +signal6 + (randn(size(signal1))+1i*randn(size(signal1))) * sqrt(1/2); % -3dB

% Select signal + noise

data=eval(['sig',letter]);

% Calculate the chirplet transform, detect significant chirplets and
% gather into chirps

[C,t,f,c,chirps,sig_chirps,selection,lmax] = chirpgathering(data,wlen,0,wlen+1,nbcr,CRmax,'h',pfa);

% Calculate the chirplet parameters for selected chirplets
hw=(wlen-1)/2;
indices=find(selection);
params=id2ids(C,find(selection));
chirplets=[t(params(2,:))'-hw , f(params(1,:))-hw*c(lmax(indices))' , wlen*ones(length(indices),1) , c(lmax(indices))'];

% Display the spectrogram (Chirplet transform at zero chirprate)
figure(1), imagesc(t,f,log(abs(C(:,:,round((nbcr+1)/2)))))
axis xy
set(gca,'fontsize',12')
ylim([0 1])
xlabel('Normalized Time')
ylabel('Normalized frequency')
title('Signal Spectrogram');

% Display the detected chirplets in TF
figure(2)
dispchirp(chirplets(:,1),chirplets(:,3),chirplets(:,2),chirplets(:,4),fs)
set(gca,'fontsize',12')
xlabel('Normalized Time')
ylabel('Normalized frequency')
title('Detected Chirplets');

% display all detected chirps
figure(3)
dispchirp(chirps(:,1),chirps(:,3)-chirps(:,1),chirps(:,2),chirps(:,4),fs)
line([0 0 4095*4 4095*4 0],[0.345 0.395 0.395 0.345 0.345], 'color', 'k')
set(gca,'fontsize',12')
xlabel('Normalized Time')
ylabel('Normalized frequency')
title('All detected chirps');

% post-processing - display only significant chirps
figure(4)
dispchirp(sig_chirps(:,1),sig_chirps(:,3)-sig_chirps(:,1),sig_chirps(:,2),sig_chirps(:,4),fs)
set(gca,'fontsize',12')
xlabel('Normalized Time')
ylabel('Normalized frequency')
title('Significant detected chirps');

% Collect the chirps together into signals
[signals]=signal_gathering(sig_chirps,t,f,CRmax);

figure(5);
dispsig(signals,fs);
set(gca,'fontsize',12')
xlabel('Normalized Time')
ylabel('Normalized frequency')
title('Grouped signals');
